%  hklcalc_v - Calculates the h,k, and l values for every pixel in an
%              image for the vertical diffractometer geometry.
%
%  [H,K,L] = hklcalc_v(Gam, Del, ov, alp, UB, k)
%
%  Input arguments:
%  ----------------
%
%  Gam,Del      2D arrays of the same size as the image data, containing
%               the gamma and delta angle values for each individual pixel.
%  ov,alp       Diffractometer angles at the position where the image was
%               taken, 1-by1 scalars.
%  UB           Orientation Matrix (i.e. UB matrix composed of the B matrix
%               used to convert the crystal lattice to a orthonormal
%               coordinate frame and the U matrix to account for sample
%               miscut).
%  k            Wave vector magnitude of the incident x-ray beam in
%               reciprocal Angstroms.
%
%  All input arguments are mandatory.
%
%  Output arguments:
%  -----------------
%
%  H            Array of the same size as Gam and Del containing the H 
%               coordinate values for each pixel.
%  K            Array of the same size as Gam and Del containing the K 
%               coordinate values for each pixel.
%  L            Array of the same size as Gam and Del containing the L 
%               coordinate values for each pixel.
%
%  Note:
%  This hklcalc_v only works for the vertical s2d3 diffractometer
%  geometry as implemented in the x04v version of SPEC at the Surface
%  Diffraction Station of the Materials Science Beamline at the Swiss
%  Light Source.

%%
%==========================================================================
% FUNCTION: hklcalc_v.m 
%           ===========
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:24:16 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/X04SA/App/lib/X_PILATUS_X04SA_Matlab/hklcalc_v.m,v $
%  
% Note:                 This code is in large parts based on a collection
%                       of macros written by Simon Oddsson Mariager,
%                       University of Copenhagen.
%  
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2006/08/15
% 
% Change Log:
% -----------
% 
% 2006/08/15 (CS):
% - start
%
% 2007/09/20 (CS):
% - renamed to hklcalc_v


%%
%==========================================================================
% Main function - HKLCALC_V
%                 =========
%
% input arguments: Gam,Del,ov,alp,UB,k
% output arguments: H, K, L
%

function [H,K,L] = hklcalc_v(Gam,Del,ov,alp,UB,k)

% Define the needed matrices. The notation follows the article by Bunk &
% Nielsen. J.Appl.Cryst. (2004) 37, 216-222.
M1 = k.*(cos(ov).*sin(Del)- ...
    sin(ov).*(cos(alp).*(cos(Gam).*cos(Del)-1)+ ...
    sin(alp).*sin(Gam).*cos(Del)));
M2 = k.*(sin(ov).*sin(Del)+ ...
    cos(ov).*(cos(alp).*(cos(Gam).*cos(Del)-1)+ ...
    sin(alp).*sin(Gam).*cos(Del)));
M3 = k.*(-sin(alp).*(cos(Gam).*cos(Del)-1)+ ...
    cos(alp).*sin(Gam).*cos(Del));

% If the matrix cannot be inverted you have a problem.
UBi = inv(UB);

% Now calculate the hkl in reciprocal lattice coordinates.
H = UBi(1,1)*M1 + UBi(2,1)*M2 + UBi(3,1)*M3;
K = UBi(1,2)*M1 + UBi(2,2)*M2 + UBi(3,2)*M3;
L = UBi(1,3)*M1 + UBi(2,3)*M2 + UBi(3,3)*M3;

%==========================================================================
%
% $Log: hklcalc_v.m,v $
% Revision 1.1  2007/10/18 15:24:16  schlepuetz
% first version
%
%
%============================= End of $RCSfile: hklcalc_v.m,v $ ===
