%  IMAGEBIN - Perform binning operation on image data.
%             
%  ImOut = imagebin(Im,binsize,<fun>)
%
%  ImOut = IMAGEBIN(Im,binsize) returns the image data <ImOut> which is
%          produced by summing all pixels of <Im> within each binning box.
%          <binsize> gives the size of the binning box [width height] in
%          number of pixels.
%          
%  ImOut = IMAGEBIN(Im,binsize,fun) uses the specified function <fun> to
%          act on the pixels within the binning box.
%          <fun> can either be a string name of a function (e.g. 'sum') or
%          a function handle (e.g. @mean or @(x) mean(x) [no quotation
%          marks!]).
%          Often used function string names could be 'sum', 'mean', or
%          'median'.
%          The function <fun> is required to act along the first
%          non-singelton dimension of a multidimensional array (such as
%          SUM, MEAN, MEDIAN, MAX, MIN, etc. do).
%
%  Example:
%          A = ones([5 7])
%          B = imagebin(A,[3 2],@(x) sum(x))
%          B = [ 6 6 ;
%                6 6 ]
%
%          B = imagebin(A,[3 2],@sum) or
%          B = imagebin(A,[3 2],'sum') would produce the same result.
%
%  Notes:
%  - The <binsize> argument is given as [width height]. The corresponding
%    matrix would be of size height-by-width. For example, a <binsize> of
%    [2 3] corresponds to an array of size 3-by-2.
%  - The binning process begins in the upper left corner of the matrix
%    (i.e. the (1,1) element) and returns only those values obtained from
%    complete binning boxes. Pixels from incomplete binning boxes are
%    dropped in the final output. For example, a 26-by-15 matrix binned
%    with a [2 3] binsize will only use the upper left 24-by-14 submatrix
%    and return an ImOut of size 8-by-7. The information in the right and
%    lower borders is lost.
%  - as this function makes use of anonymous functions, Matlab R14 or
%    higher is required.
%

%%
%==========================================================================
% FUNCTION: imagebin.m 
%           ==========
%
% Function subroutines:
% --------------------
% parse_inputs        - parse the input paramters to the main function.
%
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:18:53 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/App/lib/X_PILATUS_Matlab/imagebin.m,v $
%  
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2007/04/10
% 
%
% Change Log:
% -----------
% 
% 2007/04/10 (CS):
% - start
%
% 2007/10/17 (CS):
% - optimized routine by vectorizing the binning function call and letting
%   it act on a stack of subimages.
% - binning function can now be specified either as a function string name,
%   a function handle or by directly passing an anonymous function to
%   imagebin.

%%
%==========================================================================
% Main function - IMAGEBIN
%                 ========
%

function ImOut = imagebin(varargin)

[Im,binsize,fun] = parse_inputs(varargin{:});

% only perform binning if binsize is bigger than [1 1]
if prod(binsize)>1

    [ydim xdim] = size(Im);
    binx = binsize(1);
    biny = binsize(2);

    nbinx = floor(xdim/binx);
    nbiny = floor(ydim/biny);


    % % The slow way (but maybe safer) of doing it in a for loop over all
    % % binning boxes.
    %
    % ImOut = ones([nbiny nbinx]);
    %
    % for i = 1:nbinx
    %     for j = 1:nbiny
    %
    %         xind = (i-1)*binx+1;
    %         yind = (j-1)*biny+1;
    %         M = Im(yind:yind+biny-1,xind:xind+binx-1);
    %         M = M(:);
    %
    %         ImOut(j,i) = feval(fun,M);
    %
    %     end
    % end


    % The faster vectorized version. Requires that the function 'fun' acts
    % along the first non-singelton dimension of a multidimensional array
    % (such as SUM, MEAN, MEDIAN, MAX, MIN, etc.).

    ImStack = zeros([nbiny nbinx binx.*biny]);

    % loop over all pixels within a binning box
    ind = 1;
    for i = 1:binx
        for j = 1:biny
            ImStack(:,:,ind) = Im(j:biny:nbiny*biny,i:binx:nbinx*binx);
            ind = ind+1;
        end
    end

    % make stacking dimension the first dimension (along which the binning
    % function will be applied)
    ImStack = permute(ImStack,[3 1 2]);
    ImOut = feval(fun,ImStack);

    % remove first singelton dimension
    ImOut = squeeze(ImOut);

else
    % don't do anything if binsize was [1 1]
    ImOut = Im;
end


%%
%==========================================================================
% Sub-function - parse_inputs
% ===========================
%
%  used to parse the input paramters to the main function.

function [Im,binsize,fun] = parse_inputs(varargin)

iptchecknargin(2,3,nargin,mfilename);

Im = varargin{1};
binsize = varargin{2};

switch nargin
    
    case 2
        
        fun = @sum;
    
    case 3
        
        fun = fcnchk(varargin{3});
        
end

%%
%==========================================================================
%
% $Log: imagebin.m,v $
% Revision 1.1  2007/10/18 15:18:53  schlepuetz
% first version
%
%
%
%============================= End of $RCSfile: imagebin.m,v $ ===

