%  imagetohklint_v - calculate the H,K,L, and corrected intensity values
%                    for each pixel in an image taken in the vertical
%                    diffractometer geometry.
%
%  [H,K,L,Int] =
%      imagetohklint_v(I,gam,del,ov,alp,nu,cen,UB,k,dist,pixdim,...
%                      pperp,binsize)
%
%  Input arguments:
%  ----------------
%
%  I            Image data (2D data array).
%  gam,del,ov,alp,nu
%               Diffractometer angles at the position where the image was
%               taken.
%  cen          Position of the direct beam on the pixel detector for all
%               angles set to zero.
%  UB           Orientation Matrix (i.e. UB matrix composed of the B matrix
%               used to convert the crystal lattice to a orthonormal
%               coordinate frame and the U matrix to account for sample
%               miscut).
%  k            Wave vector magnitude of the incident x-ray beam in
%               reciprocal Angstroms.
%  dist         Distance sample - detector or detector-slits - detector in
%               mm.
%  pixdim       Dimension of an individual pixel in mm. 1-by-2 array of
%               [width height].
%  pperp        Perpendicular polarization factor, scalar between 0 and 1.
%  binsize      size of kernel in pixels for binning of the image. 1-by-2
%               array of [width height].
%
%  All input arguments are mandatory.
%
%  Output arguments:
%  -----------------
%
%  H            Array of the same size as Int containing the H coordinate
%               values for each pixel.
%  K            Array of the same size as Int containing the K coordinate
%               values for each pixel.
%  L            Array of the same size as Int containing the L coordinate
%               values for each pixel.
%  Int          Image of corrected intensity. So far, we correct for the
%               polarization factor, the intercept of the signal with the
%               Ewald sphere, exposure times and the filter transmissions.
%
%  Note:
%  This imagetohklint_v only works for the vertical s2d3 diffractometer
%  geometry as implemented in the x04v version of SPEC at the Surface
%  Diffraction Station of the Materials Science Beamline at the Swiss
%  Light Source.
%

%%
%==========================================================================
% FUNCTION: imagetohklint_v.m 
%           =================
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:24:16 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/X04SA/App/lib/X_PILATUS_X04SA_Matlab/imagetohklint_v.m,v $
%  
% Note:                 This code is in large parts based on a collection
%                       of macros written by Simon Oddsson Mariager,
%                       University of Copenhagen.
%  
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2006/08/15
% 
% Change Log:
% -----------
% 
% 2006/08/15 (CS):
% - start
%
% 2007/09/20 (CS):
% - renamed to imagetohklint_v


%%
%==========================================================================
% Main function - IMAGETOHKLINT_V
%                 ===============
%
% input arguments: I,gam,del,ov,alp,nu,cen,UB,k,dist,pixdim,pperp,binsize
% output arguments: H, K, L, Int
%

function [H,K,L,Int] = ...
    imagetohklint_v(I,gam,del,ov,alp,nu,cen,UB,k,dist,pixdim,pperp,binsize)

% convert angles to radians for subroutines
deg2rad = pi/180;
gam = gam*deg2rad;
del = del*deg2rad;
ov = ov*deg2rad;
alp = alp*deg2rad;
nu = nu*deg2rad;


[Gam,Del] = pixangles(I, gam, del, nu, cen, dist, pixdim, binsize);
[GamTemp,DelTemp] = pixangles(I, gam, del, nu, cen, dist, pixdim,[1 1]);
Int = intcorr_v(I, GamTemp, DelTemp, ov, alp, pperp);
[H, K, L] = hklcalc_v(Gam, Del, ov, alp, UB, k);

% perform binning of Int if requested
if ~all(binsize == [1 1])

    % sum over all pixels in a bin
    Int = imagebin(Int,binsize,'sum');

end

%==========================================================================
%
% $Log: imagetohklint_v.m,v $
% Revision 1.1  2007/10/18 15:24:16  schlepuetz
% first version
%
%
%============================= End of $RCSfile: imagetohklint_v.m,v $ ===

