%  intcorr_v - applies geometrical intensity corrections to each pixel of
%              an image for the vertical diffractometer geometry.
%              So far, the corrections include the polarization factor and
%              the intercept factor of the reflection with the Ewald
%              sphere.
%
%  Icorr = intcorr_v(I, Gam, Del, ov, alp, pperp)
%
%  Input arguments:
%  ----------------
%  I            Image data (2D data array).
%  Gam,Del      2D arrays of the same size as the image data, containing
%               the gamma and delta angle values for each individual pixel.
%  ov,alp       Diffractometer angles at the position where the image was
%               taken, 1-by1 scalars.
%  pperp        Perpendicular polarization factor, scalar between 0 and 1.
%
%  Output arguments:
%  -----------------
%  Icorr        2D data array containing the corrected intensity values.
%
%  Note:
%  This hklcalc_v only works for the vertical s2d3 diffractometer
%  geometry as implemented in the x04v version of SPEC at the Surface
%  Diffraction Station of the Materials Science Beamline at the Swiss
%  Light Source.

%%
%==========================================================================
% FUNCTION: intcorr_v.m 
%           ===========
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:24:16 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/X04SA/App/lib/X_PILATUS_X04SA_Matlab/intcorr_v.m,v $
%  
% Note:                 This code is in large parts based on a collection
%                       of macros written by Simon Oddsson Mariager,
%                       University of Copenhagen.
%  
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2006/08/15
% 
% Change Log:
% -----------
% 
% 2006/08/15 (CS):
% - start
%
% 2007/09/20 (CS):
% - renamed to intcorr_v


%%
%==========================================================================
% Main function -INTCORR_V
%                =========
%
% input arguments: I,Gam,Del,ov,alp,pperp
% output arguments: Icorr
%

function Icorr = intcorr_v(I,Gam,Del,ov,alp,pperp)

% calculate correction factors:

% polarization:
cp = pperp*(ones(size(Gam))-cos(Del).^2.*sin(Gam).^2) + ...
    (1-pperp).*(1-sin(Del).^2);

% intercept of scattered beam with Ewald sphere.
ci = cos(Del).*sin((Gam-alp)); 

Icorr = I.*(ci./cp);

%==========================================================================
%
% $Log: intcorr_v.m,v $
% Revision 1.1  2007/10/18 15:24:16  schlepuetz
% first version
%
%
%============================= End of $RCSfile: intcorr_v.m,v $ ===
