%  pixangles - Calculates gam and del values for each pixel in an image.
%
%  [Gam,Del] = pixangles(I, gam, del, nu, cen, dist, pixdim, binsize)
%
%
%  Input arguments:
%  ----------------
%
%  I            Image data (2D data array).
%  gam,del,nu
%               Diffractometer angles at the position where the image was
%               taken.
%  cen          Position of the direct beam on the pixel detector for all
%               angles set to zero.
%  dist         Distance sample - detector or detector-slits - detector in
%               mm.
%  pixdim       Dimension of an individual pixel in mm. 1-by-2 array of
%               [width height].
%  binsize      size of kernel in pixels for binning of the image. 1-by-2
%               array of [width height].
%
%  All input arguments are mandatory.
%
%  Output arguments:
%  -----------------
%
%  Gam          Array of the same size as I containing the gamma angle
%               values for each pixel.
%  Del          Array of the same size as I containing the gamma angle
%               values for each pixel.

%%
%==========================================================================
% FUNCTION: pixangles.m 
%           ===========
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:24:16 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/X04SA/App/lib/X_PILATUS_X04SA_Matlab/pixangles.m,v $
%  
% Note:                 This code is in large parts based on a collection
%                       of macros written by Simon Oddsson Mariager,
%                       University of Copenhagen.
%  
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2006/08/15
% 
% Change Log:
% -----------
% 
% 2006/08/15 (CS):
% - start


%%
%==========================================================================
% Main function - PIXANGLES
%                 =========
%
% input arguments: I, gam, del, nu, cen, dist, pixdim, binsize
% output arguments: Gam, Del
%

function [Gam, Del] = pixangles(I, gam, del, nu, cen, dist, pixdim,binsize)

[ydim xdim] = size(I);
[Xcoor,Ycoor] = meshgrid(1:xdim,1:ydim);

% Find the distance from cen to each pixel.
Dgam = (-Xcoor+cen(1)).*pixdim(1);       % along rows
Ddel = (-Ycoor+cen(2)).*pixdim(2);       % along columns

% perform binning if requested
if ~all(binsize == [1 1])

    Dgam = imagebin(Dgam,binsize,'mean');
    Ddel = imagebin(Ddel,binsize,'mean');

end

Radius = sqrt(Dgam.^2+Ddel.^2);

% retrieve angle of pixel with respect to cen
Angle = atan2(Dgam,Ddel);

% Now calculate the gamma and delta angles for each pixel,
% taking the nu rotation into consideration.
Anglenu = Angle-nu;
Gam = (sin(Anglenu).*(Radius./dist))+gam;
Del = (cos(Anglenu).*(Radius./dist))+del;

%==========================================================================
%
% $Log: pixangles.m,v $
% Revision 1.1  2007/10/18 15:24:16  schlepuetz
% first version
%
%
%============================= End of $RCSfile: pixangles.m,v $ ===

