%  scantohklint - calculate the H,K,L, and corrected intensity values for
%                 each pixel in all images of a scan based on the
%                 information in the corresponding scanlogfile.
%
%  [H,K,L,Int,Gam,Del] =
%      scantohklint(scanlogFile,imagePath,imageFmt,cen,k,UB,imagedim,...
%                   imagedepth,dist,pixdim,pperp,binsize,roi,geometry)
%
%  Input arguments:
%  ----------------
%
%  scanlogFile  The name of the scan log file including path.
%  imagePath    Absolute or relative path to the image directory.
%               If not supplied, the routine tries to find the image based
%               on the scanlog file name scanlogFile. This assumes a file
%               format of "image_%05d.img" and that the images are stored
%               in the path "../../pixel/images/" relative to the
%               scanlogFile.
%  imageFmt     File name format for the image files including extension.
%               This must contain a formatting string for the image number,
%               i.e. 'myname_%05d.img' (default = 'image_%05d.img').
%  cen          Position of the direct beam on the pixel detector for all
%               angles set to zero. (default = [242 89]).
%  k            Wave vector magnitude of the incident x-ray beam in
%               reciprocal Angstroms.
%               (default = 2*pi)
%  UB           Orientation Matrix (i.e. UB matrix composed of the B matrix
%               used to convert the crystal lattice to a orthonormal
%               coordinate frame and the U matrix to account for sample
%               miscut).
%               (default =  [ k 0 0
%                             0 k 0
%                             0 0 k ]
%               which yields (hkl)-coordinates in units of reciprocal
%               Angstroms.)
%  imagedim     Image dimension in pixels, 1-by-2 array [width height]
%               (default = [487 195])
%  imagedepth   Color depth of image, scalar (default = 32).
%  dist         Distance sample - detector or detector-slits - detector in
%               mm. (default = 1140.5).
%  pixdim       Dimension of an individual pixel in mm. 1-by-2 array of
%               [width height] (default = [0.172 0.172])
%  pperp        Perpendicular polarization factor, scalar between 0 and 1
%               (default = 0.98).
%  binsize      size of kernel in pixels for binning of the image. 1-by-2
%               array of [width height]. (default: no binning,
%               binsize = [1 1]).
%  roi          rectangular region of interest (ROI) in the image. Only
%               this ROI is analyzed and all output arguments will be of
%               the same size as this ROI.
%               1-by-4 array specifying [x1 y1 x2 y2], where (x1,y1) is the
%               upper left and (x2,y2) the lower right corner of the ROI
%               given in pixel units of the image.
%  geometry     diffractometer geometry which was used when taking the
%               scans: Can be 'vertical' or 'horizontal'
%               (default = 'vertical').
%
%  All arguments besides scanlogFile are optional. If not supplied or left
%  empty (i.e. []), the default values will be used.
%
%  Output arguments:
%  -----------------
%
%  H            Array of the same size as Int containing the H coordinate
%               values for each pixel.
%  K            Array of the same size as Int containing the K coordinate
%               values for each pixel.
%  L            Array of the same size as Int containing the L coordinate
%               values for each pixel.
%  Int          Image of corrected intensity. So far, we correct for the
%               polarization factor, the intercept of the signal with the
%               Ewald sphere, exposure times and the filter transmissions.

%%
%==========================================================================
% FUNCTION: scantohklint.m
%           ==============
%
% CVS-Info:
% ---------
% $Date: 2007/10/18 15:24:16 $
% $Author: schlepuetz $
% $Revision: 1.1 $
% $Source: /cvs/X/PILATUS/X04SA/App/lib/X_PILATUS_X04SA_Matlab/scantohklint.m,v $
%
% Note:                 This code is in large parts based on a collection
%                       of macros written by Simon Oddsson Mariager,
%                       University of Copenhagen.
%
% Author(s):            C.M. Schlepuetz (CS)
% Co-author(s):         Stephan Pauli (SP)
% Address:              Surface Diffraction Station
%                       Materials Science Beamline X04SA
%                       Swiss Light Source (SLS)
%                       Paul Scherrer Institut
%                       CH - 5232 Villigen PSI
% Created and (c):      2006/09/01
%
% Change Log:
% -----------
%
% 2006/09/01 (CS):
% - start
%
% 2007/09/20 (CS):
% - adapted the code such that either vertical or horizontal geometry scans
%   can be analyzed.
%
% 2007/10/11 (CS):
% - changed default of UB matrix from unit matrix to
%     [ k 0 0
%       0 k 0
%       0 0 k ]
%   which directly yields the (hkl)-coordinates in units of reciprocal
%   Anstroms.
% - as a consequence, order of input parameters k and UB was changed.



%%
%==========================================================================
% Main function - SCANTOHKLINT
%                 ============
%
% input arguments:  scanlogFile,imagePath,cen,k,UB,imagedim,...
%                   imagedepth,dist,pixdim,pperp,binsize,roi,geometry
% output arguments: H,K,L,Int
%

function [H,K,L,Int] = scantohklint(varargin)

[scanlogFile,imagePath,imageFmt,cen,k,UB,imext,imagedim,imagedepth,...
    dist,pixdim,pperp,binsize,roi,geometry] = ...
    parse_inputs(varargin{:});

[scanlogdata,scanlogheader]=readscanlog(scanlogFile);
data_dim = size(scanlogdata);
npoints = data_dim(1);

% find columns containing angle values and image numbers
if strmatch('ID', scanlogheader','exact')
    imgcol = scanlogdata(:,strmatch('ID', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find image number column.\n');
end
if strmatch('gam', scanlogheader','exact')
    gamcol = scanlogdata(:,strmatch('gam', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find gamma value column.\n');
end
if strmatch('del', scanlogheader','exact')
    delcol = scanlogdata(:,strmatch('del', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find delta value column.\n');
end

switch geometry

    case 'vertical'
        if strmatch('alp', scanlogheader','exact')
            alpcol = scanlogdata(:,strmatch('alp', scanlogheader','exact'));
        else
            fprintf(1,'\nCould not find alpha value column.\n');
        end
        if strmatch('ov', scanlogheader','exact')
            ovcol = scanlogdata(:,strmatch('ov', scanlogheader','exact'));
        else
            fprintf(1,'\nCould not find omagaV value column.\n');
        end

    case 'horizontal'
        if strmatch('oh', scanlogheader','exact')
            ohcol = scanlogdata(:,strmatch('oh', scanlogheader','exact'));
        else
            fprintf(1,'\nCould not find alpha value column.\n');
        end
        if strmatch('phi', scanlogheader','exact')
            phicol = scanlogdata(:,strmatch('phi', scanlogheader','exact'));
        else
            fprintf(1,'\nCould not find omagaV value column.\n');
        end

end

if strmatch('nu', scanlogheader','exact')
    nucol = scanlogdata(:,strmatch('nu', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find nu value column.\n');
end
if strmatch('transm.', scanlogheader','exact')
    transmcol = scanlogdata(:,strmatch('transm.', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find transmission value column.\n');
end
if strmatch('sec', scanlogheader','exact')
    seccol = scanlogdata(:,strmatch('sec', scanlogheader','exact'));
else
    fprintf(1,'\nCould not find exposure time column.\n');
end

%%
% preallocate arrays for speed

% read first image
imfilename = strcat(imagePath,filesep,sprintf(imageFmt,imgcol(1)));
I1 = imageread(imfilename, imext, imagedim, imagedepth);

% apply roi
if ~isempty(roi)
    I1 = I1(roi(2):roi(4),roi(1):roi(3));
end

% perform binning
if ~all(binsize == [1 1])
    I1 = imagebin(I1,binsize,'mean');
end

bindim = size(I1);
Int = zeros([bindim(1) bindim(2) npoints]);
H = zeros([bindim(1) bindim(2) npoints]);
K = zeros([bindim(1) bindim(2) npoints]);
L = zeros([bindim(1) bindim(2) npoints]);


% redefine the center of the detector if a roi is used
if ~isempty(roi)
    cen = cen - [roi(1)-1 roi(2)-1];
end

%%
% loop over all scan points

fprintf(1,'\nscan has %d datapoints: \n',npoints)
fprintf(1,'processing: \ninit')

for i=1:npoints

    fprintf(1,'\b\b\b\b')
    fprintf(1,'%4d',i)

    imfilename = strcat(imagePath,filesep,sprintf(imageFmt,imgcol(i)));
    I = imageread(imfilename, imext, imagedim, imagedepth);

    if ~isempty(roi)
        I = I(roi(2):roi(4),roi(1):roi(3));
    end

    switch geometry
        case 'vertical'
            [H1,K1,L1,Int1] = imagetohklint_v(I,gamcol(i),delcol(i),...
                ovcol(i),alpcol(i),nucol(i),cen,UB,k,dist,pixdim,pperp,...
                binsize);

        case 'horizontal'
            [H1,K1,L1,Int1] = imagetohklint_h(I,gamcol(i),delcol(i),...
                phicol(i),ohcol(i),nucol(i),cen,UB,k,dist,pixdim,pperp,...
                binsize);
    end

    H(:,:,i) = H1;
    K(:,:,i) = K1;
    L(:,:,i) = L1;
    Int(:,:,i) = Int1 ./ transmcol(i) ./ seccol(i);

end
fprintf(1,'\ndone\n\n')

%%
%==========================================================================
%  Sub-function - parse_inputs
%  ===========================
%
%  used to parse the input paramters to the main function.
%  input arguments:  scanlogFile, [imagePath,cen,k,UB,dist,pixdim,pperp]
function [scanlogFile,imagePath,imageFmt,cen,k,UB,imext,imagedim,...
    imagedepth,dist,pixdim,pperp,binsize,roi,geometry] = ...
    parse_inputs(varargin)

%Check number of input arguments, are there 1 to 10.
error(nargchk(1, 14, nargin));

% check if scanlogFile exists
scanlogFile = char(varargin{1});
if (~(exist(scanlogFile,'file')))
    eid = sprintf('Images:%s:FileDoesNotExist',mfilename);
    error(eid,'scanlog file does not exist: %s.',scanlogFile);
end

% initialize optional arguments if they have not been given.
if nargin < 14; geometry = [];      else geometry = varargin{14};   end
if nargin < 13; roi = [];           else roi = varargin{13};        end
if nargin < 12; binsize = [];       else binsize = varargin{12};    end
if nargin < 11; pperp = [];         else pperp = varargin{11};      end
if nargin < 10; pixdim = [];        else pixdim = varargin{10};     end
if nargin < 9;  dist = [];          else dist = varargin{9};        end
if nargin < 8;  imagedepth = [];    else imagedepth = varargin{8};  end
if nargin < 7;  imagedim = [];      else imagedim = varargin{7};    end
if nargin < 6;  UB = [];            else UB = varargin{6};          end
if nargin < 5;  k = [];             else k = varargin{5};           end
if nargin < 4;  cen = [];           else cen = varargin{4};         end
if nargin < 3;  imageFmt = [];      else imageFmt = varargin{3};    end
if nargin < 2;  imagePath = [];     else imagePath = varargin{2};   end

% assign default values to optional arguments which have not been assigned.
if isempty(geometry);  geometry = 'vertical';       end
if isempty(binsize);  binsize = [1 1];              end
if isempty(pperp);  pperp = 0.98;                   end
if isempty(pixdim);  pixdim = [0.172 0.172];        end
if isempty(dist);  dist = 1140.5;                   end
if isempty(imagedepth);  imagedepth = 32;           end
if isempty(imagedim);  imagedim = [487 195];        end
if isempty(k);  k = 2*pi;                           end
if isempty(UB);  UB = eye(3).*k;                    end
if isempty(cen);  cen = [242 89];                   end
if isempty(imageFmt);  imageFmt = 'image_%05d.img'; end

% check if roi array has correct dimension
if ~isempty(roi)
    if (any(size(roi) ~= [1 4]))
        eid = sprintf('Images:%s:WrongArraySize',mfilename);
        error(eid,'roi must be a 1-by-4 array');
    end
end

% check if binsize array has correct dimension
if (any(size(binsize) ~= [1 2]))
    eid = sprintf('Images:%s:WrongArraySize',mfilename);
    error(eid,'binsize must be a 1-by-2 array');
end

% check if UB matrix has correct dimension
if (any(size(UB) ~= [3 3]))
    eid = sprintf('Images:%s:WrongMatrixSize',mfilename);
    error(eid,'UB matrix must be a 3-by-3 array');
end

% if imagePath is not given try to reconstruct from scanlogFile, check if
% exists.
imagePath = char(imagePath);
if isempty(imagePath)
    [logpath] = fileparts(scanlogFile);
    exppath = fileparts(fileparts(logpath));
    imagePath = [exppath filesep 'pixel' filesep 'images' filesep];
end

if (~(exist(imagePath,'dir')))
    eid = sprintf('Images:%s:FileDoesNotExist',mfilename);
    error(eid,'Could not find image path: %s.\n%s',imagePath,...
        'Please specify an image path as a command argument');
end

% find out image format (.img, .tif, .edf)
dots = findstr(imageFmt, '.');
imext = imageFmt(dots(end)+1:end);

% check if geometry argument is valid
if ~any(strcmp({'vertical' 'horizontal'}, geometry))
    eid = sprintf('Images:%s:InvalidGeometry',mfilename);
    error(eid,'%s %s %s',...
        'Invalid argument for ''geometry'' option in ',...
        mfilename,...
        '. Valid arguments are ''vertical'' and ''horizontal''');
end


%==========================================================================
%
% $Log: scantohklint.m,v $
% Revision 1.1  2007/10/18 15:24:16  schlepuetz
% first version
%
%
%============================= End of $RCSfile: scantohklint.m,v $ ===







